//******************************************************
//
//  The Emulator Of SAA1099 Sound Chip
//  copyright (c) Aley Keprt, 1996-1999
//  this is C++ version
//
//  version 0.60
//
//******************************************************


#ifndef __SAAEMU_H
#define __SAAEMU_H


#ifndef BYTE
 typedef unsigned char BYTE;
 typedef unsigned short WORD;
 typedef unsigned long DWORD;
#endif


enum SAA_Device {
 DEV_Auto=-1,
 DEV_None=0,
 DEV_OPL2,
 DEV_OPL3,
 DEV_GUS,
 DEV_HOOPER,
};


//SAA registers
typedef struct {
 BYTE vol[8];    //00-05 volume
 BYTE freq[8];   //08-13 frequency
 BYTE oct[4];    //16-19 octave
 BYTE enable[2]; //20,21 channel-enable
 BYTE noise;     //22    noise genarator
 BYTE reg23;
 BYTE env[4];    //24,25 envelope
 BYTE ctrl;      //28    output control
 BYTE reg29;
 BYTE reg30;
 BYTE reg31;
} TSAAregs;


//SAA emulator's infotable
typedef struct {
 union {
  TSAAregs sreg;          //SAA hardware registers
  BYTE reg[32];
 };

 DWORD __reserved__;
 WORD ver;                //current version
 WORD oldver;             //lowest compatible version
 BYTE chmask;             //bit-masks of channels
                          //bit 0 set masks channel 0 etc.
 BYTE curmode;            //current sound device mode
 BYTE maxmodes;           //number of sound device modes
} TSAAInfoStruct;



class SaaEmul {

public:

 TSAAInfoStruct SAAtab;

 void UpdateSound(); //called 50 times per second

 int Init(SAA_Device device=DEV_Auto,int port=0); //0=ok, else error

 void Reset(); //reset sound device
 
 void Close(); //close sound device (call this on exit)

 const char* GetName()const; //get player name

 const char* DeviceName()const; //get current audio driver name

 int SetMode(int mode); //set audio mode

 int GetVolume(); //get volume (always 0..31)

 void SetVolume(int vol); //set volume (0..31)

 void DirectOut(int reg,int val);
};


#endif
